@@ -1,7 +1,7 @@
 /*
 	ccid_usb.c: USB access routines using the libusb library
 	Copyright (C) 2003-2010	Ludovic Rousseau
-	Copyright (C) 2009-2018	Advanced Card Systems Ltd.
+	Copyright (C) 2009-2019	Advanced Card Systems Ltd.
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Lesser General Public
@@ -109,6 +109,7 @@ typedef struct
 #ifdef __APPLE__
 	// Thread handle for card detection
 	pthread_t hThread;
+	int threadCreated;
 
 	// Flag to terminate thread
 	int terminated;
@@ -660,19 +661,14 @@ again:
 					 * So on Mac OS X the reader configuration is not done
 					 * by the OS/kernel and we do it ourself.
 					 */
-					if ((0xFF == desc.bDeviceClass)
-						&& (0xFF == desc.bDeviceSubClass)
-						&& (0xFF == desc.bDeviceProtocol))
+					r = libusb_set_configuration(dev_handle, 1);
+					if (r < 0)
 					{
-						r = libusb_set_configuration(dev_handle, 1);
-						if (r < 0)
-						{
-							(void)libusb_close(dev_handle);
-							DEBUG_CRITICAL4("Can't set configuration on %d/%d: %s",
-									bus_number, device_address,
-									libusb_error_name(r));
-							continue;
-						}
+						(void)libusb_close(dev_handle);
+						DEBUG_CRITICAL4("Can't set configuration on %d/%d: %s",
+								bus_number, device_address,
+								libusb_error_name(r));
+						continue;
 					}
 
 					/* recall */
@@ -1043,7 +1039,10 @@ again:
 				usbDevice[reader_index].pTerminated = &usbDevice[reader_index].terminated;
 				usbDevice[reader_index].pTransfer = &usbDevice[reader_index].polling_transfer;
 				usbDevice[reader_index].pTransferLock = &usbDevice[reader_index].transferLock;
+				usbDevice[reader_index].threadCreated = FALSE;
 				usbDevice[reader_index].ccid.pbStatusLock = &usbDevice[reader_index].ccid.bStatusLock;
+				usbDevice[reader_index].ccid.lastSlotOpened = FALSE;
+				usbDevice[reader_index].ccid.pLastSlotOpened = &usbDevice[reader_index].ccid.lastSlotOpened;
 
 				// Create transfer lock
 				r = pthread_mutex_init(usbDevice[reader_index].pTransferLock, NULL);
@@ -1068,6 +1067,15 @@ again:
 					goto end2;
 				}
 
+				/* Disable card detection thread for APG8201 series. */
+				if ((usbDevice[reader_index].ccid.readerID == ACS_APG8201)
+					|| (usbDevice[reader_index].ccid.readerID == ACS_APG8201_B2)
+					|| (usbDevice[reader_index].ccid.readerID == ACS_APG8201Z)
+					|| (usbDevice[reader_index].ccid.readerID == ACS_APG8201Z2))
+				{
+					goto end;
+				}
+
 				// Create thread for card detection
 				r = pthread_create(&usbDevice[reader_index].hThread, NULL, CardDetectionThread, (void *) (intptr_t) reader_index);
 				if (r != 0)
@@ -1081,6 +1089,8 @@ again:
 					return_value = STATUS_UNSUCCESSFUL;
 					goto end2;
 				}
+
+				usbDevice[reader_index].threadCreated = TRUE;
 #endif
 				goto end;
 			}
@@ -1365,29 +1375,33 @@ status_t CloseUSB(unsigned int reader_index)
 		DEBUG_COMM("Last slot closed. Release resources");
 
 #ifdef __APPLE__
-		DEBUG_INFO3("Terminating thread: %d/%d",
-			usbDevice[reader_index].bus_number,
-			usbDevice[reader_index].device_address);
-
-		// Terminate thread
-		*usbDevice[reader_index].pTerminated = TRUE;
-
-		// Lock transfer
-		pthread_mutex_lock(usbDevice[reader_index].pTransferLock);
-
-		// Cancel transfer
-		if (*usbDevice[reader_index].pTransfer != NULL)
+		if (usbDevice[reader_index].threadCreated)
 		{
-			libusb_cancel_transfer(*usbDevice[reader_index].pTransfer);
-			*usbDevice[reader_index].pTransfer = NULL;
+			DEBUG_INFO3("Terminating thread: %d/%d",
+				usbDevice[reader_index].bus_number,
+				usbDevice[reader_index].device_address);
+
+			// Terminate thread
+			*usbDevice[reader_index].pTerminated = TRUE;
+
+			// Lock transfer
+			pthread_mutex_lock(usbDevice[reader_index].pTransferLock);
+
+			// Cancel transfer
+			if (*usbDevice[reader_index].pTransfer != NULL)
+			{
+				libusb_cancel_transfer(*usbDevice[reader_index].pTransfer);
+				*usbDevice[reader_index].pTransfer = NULL;
+			}
+
+			// Unlock transfer
+			pthread_mutex_unlock(usbDevice[reader_index].pTransferLock);
+
+			// Wait thread
+			pthread_join(usbDevice[reader_index].hThread, NULL);
+			usbDevice[reader_index].threadCreated = FALSE;
 		}
 
-		// Unlock transfer
-		pthread_mutex_unlock(usbDevice[reader_index].pTransferLock);
-
-		// Wait thread
-		pthread_join(usbDevice[reader_index].hThread, NULL);
-
 		// Free bStatus lock
 		pthread_mutex_destroy(usbDevice[reader_index].ccid.pbStatusLock);
 
