@@ -17,11 +17,6 @@
 	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/*
- * $Id$
- */
-
-
 #include <config.h>
 #include "misc.h"
 #include "debug.h"
@@ -48,7 +43,7 @@
 
 void log_msg(const int priority, const char *fmt, ...)
 {
-	char debug_buffer[160]; /* up to 2 lines of 80 characters */
+	char debug_buffer[3 * 80]; /* up to 3 lines of 80 characters */
 	va_list argptr;
 	static struct timeval last_time = { 0, 0 };
 	struct timeval new_time = { 0, 0 };
